<?php

namespace Drupal\hubspot_integration\Services;

use Drupal\Core\Database\Connection;
use Symfony\Component\HttpFoundation\Session\Session;
use Drupal\Core\Config\ConfigFactory;

class HubspotAPI {
  /*
   * @var \Drupal\Core\Database\Connection $database
   */
  protected $database;
  /*
   * @var \Symfony\Component\HttpFoundation\Session\Session $session
   */
  protected $session;
  /*
   * @var \Drupal\Core\Config\ConfigFactory $config_factory
   */
  protected $config_factory;
  
  /**
   * Constructs a new HubspotAPI object.
   * @param \Drupal\Core\Database\Connection $connection
   * @param \Symfony\Component\HttpFoundation\Session\Session $session
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   */
  public function __construct(Connection $connection, Session $session, ConfigFactory $config_factory) {
    $this->database = $connection;
    $this->session = $session;
    $this->config_factory = $config_factory;
  }
  
  /**
   * Get Hubspot hubspotutk cookie value
   * @return mixed
   */
  public function getCookie() {
    return \Drupal::request()->cookies->get('hubspotutk');
  }
  
  /**
   * Get current session
   */
  public function getSession() {
    return $this->session;
  }
  
  /**
   * Determine if current user is registered as a contact on husbpot
   * @return unknown
   */
  public function isContact() {
    $user = $this->getContactInfo();
    $property = 'is-contact';
    return $user->{$property};
  }
  
  /**
   * Get Husbspot values for the current user
   * 
   * @param boolean $forced_update
   * @return NULL|mixed
   */
  public function getContactInfo($forced_update=false) {
    $user = NULL;
    if (!$this->isCacheDisabled() && $this->session->get('hubspot_integration.user') && !$forced_update) {
      $user = $this->session->get('hubspot_integration.user');
    }
    else {
      $cookie = $this->getCookie();
      if(!empty($cookie)) {
        $user = $this->request("contacts/v1/contact/utk/$cookie/profile");
        $this->session->set('hubspot_integration.user', $user);
      }
    }
    return $user;
  }
  
  /**
   * Get the config name for the hubspot integration module settings
   * 
   * @param unknown $name
   * @return mixed|NULL|array|\Drupal\Component\Render\MarkupInterface|string|unknown[]|array[]
   */
  public function getConfig($name) {
    return $this->config_factory->get('hubspot_integration.settings')->get($name);
  }
  
  /**
   * Get the list of the possible contact properties and their definitions
   * 
   * @return mixed
   */
  public function getContactProperties() {
    return $this->request('properties/v1/contacts/properties');
  }
  
  /**
   * Function to make calls on hubspot
   * 
   * @param unknown $api
   * @return mixed
   */
  protected function request($api) {
    $api_key = $this->getConfig('api_key');
    $hub_url = 'https://api.hubapi.com/';
    
    $url = 'https://api.hubapi.com/'.$api."?hapikey=$api_key";
    $json = file_get_contents($url);
    $ret = json_decode($json);
    return $ret;
  }
  
  /**
   * Get the mapping done between Drupal and Hubspot
   * @return mixed
   */
  public function getMapping() {
    return $this->config_factory->get('hubspot_integration.mapping')->get('mapping');;
  }
  
  /**
   * Utility function to transform a text as a machine name
   * 
   * @param unknown $string
   * @return string
   */
  public static function machineName($string) {
    return preg_replace('@[^a-z0-9-]+@','-', strtolower($string));
  }
  
  /**
   * Utility function to check a vocabulary is mapped with hubspot
   * @param unknown $vocabulary
   * @return boolean
   */
  public function isVocabularyMapped($vocabulary) {
    $mapping = $this->getMapping();
    
    foreach($mapping as $mapping) {
      if (isset($mapping['#vocabulary']) && $mapping['#vocabulary']==$vocabulary) {
        return true;
      }
    }
    
    return false;
  }
  
  public function isCacheDisabled() {
    return $this->getConfig('cache_disabled');
  }
  
}